<?php
if ($argc != 5) {
    echo("Syntax: ".__FILE__." CURRENCY_PAIR START_DATE END_DATE OUT_FILE\nSTART_DATE and END_DATE must be of the form YYYYMM\nExample: ".__FILE__." EURUSD 200702 200908 out.csv\n");
    exit(1);
}
$pair = $argv[1];
if (!file_exists($pair)) {
    echo "Error: $pair folder does not exist\n";
    exit(1);
}
if (strlen($argv[2]) != 6 || strlen($argv[3]) != 6) {
    echo "The input dates must be of the form YYYYMM, example 200808\n";
    exit(1);
}
//
// open files
//
$outfile = $argv[4].".csv" ;
$askfile = "$".$argv[4].".Ask.txt" ;
$bidfile = "$".$argv[4].".Bid.txt" ;
$startyear = substr($argv[2],0,4);
$startmonth = substr($argv[2],4,2);
$endyear = substr($argv[3],0,4);
$endmonth = substr($argv[3],4,2);
if ($startyear < 1900 || $startyear > 2100 || $endyear < 1900 || $endyear > 2100 || $startmonth < 1 || $startmonth > 12 || $endmonth < 1 || $endmonth > 12) {
    echo "The input dates must be of the form YYYYMM, example 200808\n";
    exit(1);
}
$outfd = fopen($outfile,'w');
if ($outfd === FALSE) {
    echo "Cannot open $outfile for writing.\n";
    exit(1);
}
$askfd = fopen($askfile,'w');
if ($askfd === FALSE) {
    echo "Cannot open $askfd for writing.\n";
    exit(1);
}
$bidfd = fopen($bidfile,'w');
if ($outfd === FALSE) {
    echo "Cannot open $bidfd for writing.\n";
    exit(1);
}

$starttime = gmmktime(0,0,0,$startmonth,1,$startyear);
$endtime = gmmktime(0,0,0,$endmonth,1,$endyear);
print "Processing data starting from ".gmstrftime("%D %T",$starttime)." up to ".gmstrftime("%D %T",$endtime)."\n";
$starttime -= $starttime % 3600;
for($i = $starttime; $i < $endtime; $i += 3600) {
    $year = gmstrftime('%Y',$i);
    $month = str_pad(gmstrftime('%m',$i) - 1, 2, '0', STR_PAD_LEFT);
    $day = gmstrftime('%d',$i);
    $hour = gmstrftime('%H',$i);
    $localpath = "$pair/$year/$month/$day/";
    $localfile = $localpath . $hour . "h_ticks.bin";
    if (!file_exists($localpath)) {
        echo "Error: $localpath does not exist\n";
    }
    if (file_exists($localfile)) {
        if (filesize($localfile) > 0) {
            decode_ducascopy_bin($localfile, $outfd , $bidfd ,$askfd);
        }
        else {
            echo "Warning: 0 sized $localfile\n";
        }
    }
    else {
        echo "Warning: $localfile does not exist\n";
    }
}
fclose($outfd);
fclose($askfd);
fclose($bidfd);

function decode_ducascopy_bin($fname, $outfd, $bidfd, $askfd ) {
    print "$fname\n";
    $zip = new ZipArchive;
    $res = $zip->open($fname);
    $binname = $zip->getNameIndex(0);
    if ($res!==true) {
        echo "Error: failed to open [$fname] code [$res]\n";
        exit(1);
    }
    $fp = $zip->getStream($binname);
    if (!$fp) {
        echo "Error: unable to open zip stream\n";
        exit(1);
    }
    $bin = stream_get_contents($fp);
    $idx = 0;
    $size = strlen($bin);
    while($idx < $size) {
        //print "$idx $size\n";
        $q = unpack('@'.$idx.'/n4', $bin);
        $time = bcmul('4294967296', bcadd($q['2'],bcmul($q['1'],65536)));
        $time = bcadd($time, bcadd($q['4'],bcmul($q['3'],65536)));
        $timesec = bcdiv($time, 1000);
        $timems = bcmod($time, 1000);

        $q = unpack('@'.($idx + 8)."/N2", $bin);
        $s = pack('V2', $q[2], $q[1]);
        $q = unpack('d', $s);
        $ask = $q[1];

        $q = unpack('@'.($idx + 16)."/N2", $bin);
        $s = pack('V2', $q[2], $q[1]);
        $q = unpack('d', $s);
        $bid = $q[1];

        $q = unpack('@'.($idx + 24)."/N2", $bin);
        $s = pack('V2', $q[2], $q[1]);
        $q = unpack('d', $s);
        $askvol = $q[1];

        $q = unpack('@'.($idx + 32)."/N2", $bin);
        $s = pack('V2', $q[2], $q[1]);
        $q = unpack('d', $s);
        $bidvol = $q[1];

        fwrite($outfd, gmstrftime("%Y.%m.%d %H:%M:%S", $timesec).".".str_pad($timems,3,'0',STR_PAD_LEFT).",$bid,$ask,".number_format($bidvol,0,'','').",".number_format($askvol,0,'','')."\n");
        fwrite($bidfd, gmstrftime("%Y%m%d %H%M%S", $timesec).";$bid;".number_format($bidvol,0,'','')."\n");
        fwrite($askfd, gmstrftime("%Y%m%d %H%M%S", $timesec).";$ask;".number_format($askvol,0,'','')."\n");

        $idx += 40;
    }
}
